@@ -630,7 +630,8 @@ graphene_matrix_to_2d (const graphene_matrix_t *m,
   float res[4];
 
   graphene_simd4f_dup_4f (m->value.x, res);
-  if (res[2] != 0.f && res[3] != 0.f)
+  if (!graphene_approx_val (res[2], 0.f) &&
+      !graphene_approx_val (res[3], 0.f))
     return false;
 
   if (xx != NULL)
@@ -639,7 +640,8 @@ graphene_matrix_to_2d (const graphene_matrix_t *m,
     *yx = res[1];
 
   graphene_simd4f_dup_4f (m->value.y, res);
-  if (res[2] != 0.f && res[3] != 0.f)
+  if (!graphene_approx_val (res[2], 0.f) &&
+      !graphene_approx_val (res[3], 0.f))
     return false;
 
   if (xy != NULL)
@@ -647,11 +649,16 @@ graphene_matrix_to_2d (const graphene_matrix_t *m,
   if (yy != NULL)
     *yy = res[1];
 
-  if (graphene_simd4f_cmp_neq (m->value.z, graphene_simd4f_init (0.f, 0., 1.f, 0.f)))
+  graphene_simd4f_dup_4f (m->value.z, res);
+  if (!graphene_approx_val (res[0], 0.f) &&
+      !graphene_approx_val (res[1], 0.f) &&
+      !graphene_approx_val (res[2], 1.f) &&
+      !graphene_approx_val (res[3], 0.f))
     return false;
 
   graphene_simd4f_dup_4f (m->value.w, res);
-  if (res[2] != 0.f && res[3] != 1.f)
+  if (!graphene_approx_val (res[2], 0.f) &&
+      !graphene_approx_val (res[3], 1.f))
     return false;
 
   if (x_0 != NULL)
@@ -746,7 +753,25 @@ graphene_matrix_get_value (const graphene_matrix_t *m,
       return 0.f;
     }
 
-  return graphene_simd4f_get (r, col);
+  switch (col)
+    {
+    case 0:
+      return graphene_simd4f_get (r, 0);
+
+    case 1:
+      return graphene_simd4f_get (r, 1);
+
+    case 2:
+      return graphene_simd4f_get (r, 2);
+
+    case 3:
+      return graphene_simd4f_get (r, 3);
+
+    default:
+      return 0.f;
+    }
+
+  return 0.f;
 }
 
 /**
@@ -903,10 +928,10 @@ graphene_matrix_transform_rect (const graphene_matrix_t *m,
   graphene_simd4f_t __s; \
   graphene_point_t __p; \
   graphene_rect_get_ ## corner (rect, &__p); \
-  __s = graphene_simd4f_init (__p.x, __p.y, 0.f, 0.f); \
-  graphene_simd4x4f_vec3_mul (&matrix->value, &__s, &__s); \
-  out_p.x = graphene_simd4f_get (__s, 0); \
-  out_p.y = graphene_simd4f_get (__s, 1);           } while (0)
+  __s = graphene_simd4f_init (__p.x, __p.y, 0.f, 1.f); \
+  graphene_simd4x4f_vec4_mul (&matrix->value, &__s, &__s); \
+  out_p.x = graphene_simd4f_get_x (__s); \
+  out_p.y = graphene_simd4f_get_y (__s);           } while (0)
 
   TRANSFORM_POINT (m, r, top_left, ret[0]);
   TRANSFORM_POINT (m, r, top_right, ret[1]);
@@ -943,10 +968,10 @@ graphene_matrix_transform_bounds (const graphene_matrix_t *m,
   graphene_simd4f_t __s; \
   graphene_point_t __p; \
   graphene_rect_get_ ## corner (rect, &__p); \
-  __s = graphene_simd4f_init (__p.x, __p.y, 0.f, 0.f); \
-  graphene_simd4x4f_vec3_mul (&matrix->value, &__s, &__s); \
-  out_p.x = graphene_simd4f_get (__s, 0); \
-  out_p.y = graphene_simd4f_get (__s, 1);           } while (0)
+  __s = graphene_simd4f_init (__p.x, __p.y, 0.f, 1.f); \
+  graphene_simd4x4f_vec4_mul (&matrix->value, &__s, &__s); \
+  out_p.x = graphene_simd4f_get_x (__s); \
+  out_p.y = graphene_simd4f_get_y (__s);           } while (0)
 
   TRANSFORM_POINT (m, r, top_left, ret[0]);
   TRANSFORM_POINT (m, r, top_right, ret[1]);
