@@ -27,6 +27,7 @@
 #define _GNU_SOURCE
 
 #include "config.h"
+#include "graphene-macros.h"
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
@@ -101,6 +102,12 @@
 
 #endif /* __GNUC__ */
 
+static inline bool
+graphene_approx_val (float a, float b)
+{
+  return graphene_fuzzy_equals (a, b, FLT_EPSILON);
+}
+
 static inline float
 graphene_flerpf (float a, float b, double factor)
 {
