@@ -229,7 +229,8 @@ graphene_vec2_add (const graphene_vec2_t *a,
  * @b: a #graphene_vec2_t
  * @res: (out caller-allocates): return location for the result
  *
- * Subtracts each component of the two passed vectors and places
+ * Subtracts from each component of the first operand @a the
+ * corresponding component of the second operand @b and places
  * each result into the components of @res.
  *
  * Since: 1.0
@@ -267,8 +268,9 @@ graphene_vec2_multiply (const graphene_vec2_t *a,
  * @b: a #graphene_vec2_t
  * @res: (out caller-allocates): return location for the result
  *
- * Divides each component of the two passed vectors and places
- * each result into the components of @res.
+ * Divides each component of the first operand @a by the corresponding
+ * component of the second operand @b, and places the results into the
+ * vector @res.
  *
  * Since: 1.0
  */
@@ -395,7 +397,7 @@ graphene_vec2_scale (const graphene_vec2_t *v,
  * @v: a #graphene_vec2_t
  * @res: (out caller-allocates): return location for the result vector
  *
- * Negates the give #graphene_vec2_t.
+ * Negates the given #graphene_vec2_t.
  *
  * Since: 1.2
  */
@@ -803,7 +805,9 @@ graphene_vec3_add (const graphene_vec3_t *a,
  * @b: a #graphene_vec3_t
  * @res: (out caller-allocates): return location for the resulting vector
  *
- * Subtracts each component of the two given vectors.
+ * Subtracts from each component of the first operand @a the
+ * corresponding component of the second operand @b and places
+ * each result into the components of @res.
  *
  * Since: 1.0
  */
@@ -839,7 +843,9 @@ graphene_vec3_multiply (const graphene_vec3_t *a,
  * @b: a #graphene_vec3_t
  * @res: (out caller-allocates): return location for the resulting vector
  *
- * Divides each component of the two given vectors.
+ * Divides each component of the first operand @a by the corresponding
+ * component of the second operand @b, and places the results into the
+ * vector @res.
  *
  * Since: 1.0
  */
@@ -884,7 +890,7 @@ float
 graphene_vec3_dot (const graphene_vec3_t *a,
                    const graphene_vec3_t *b)
 {
-  return graphene_simd4f_get_x (graphene_simd4f_dot3 (a->value, b->value));
+  return graphene_simd4f_dot3_scalar (a->value, b->value);
 }
 
 /**
@@ -1072,7 +1078,7 @@ graphene_vec3_get_xyzw (const graphene_vec3_t *v,
  * @v: a #graphene_vec3_t
  * @res: (out caller-allocates): return location for the result vector
  *
- * Negates the give #graphene_vec3_t.
+ * Negates the given #graphene_vec3_t.
  *
  * Since: 1.2
  */
@@ -1139,7 +1145,7 @@ graphene_vec3_near (const graphene_vec3_t *v1,
 
   d = graphene_simd4f_sub (v1->value, v2->value);
 
-  return graphene_simd4f_get_x (graphene_simd4f_dot3 (d, d)) < epsilon_sq;
+  return graphene_simd4f_dot3_scalar (d, d) < epsilon_sq;
 }
 
 enum {
@@ -1570,7 +1576,9 @@ graphene_vec4_add (const graphene_vec4_t *a,
  * @b: a #graphene_vec4_t
  * @res: (out caller-allocates): return location for the resulting vector
  *
- * Subtracts each component of the two given vectors.
+ * Subtracts from each component of the first operand @a the
+ * corresponding component of the second operand @b and places
+ * each result into the components of @res.
  *
  * Since: 1.0
  */
@@ -1606,7 +1614,9 @@ graphene_vec4_multiply (const graphene_vec4_t *a,
  * @b: a #graphene_vec4_t
  * @res: (out caller-allocates): return location for the resulting vector
  *
- * Divides each component of the two given vectors.
+ * Divides each component of the first operand @a by the corresponding
+ * component of the second operand @b, and places the results into the
+ * vector @res.
  *
  * Since: 1.0
  */
@@ -1767,7 +1777,7 @@ graphene_vec4_get_xyz (const graphene_vec4_t *v,
  * @v: a #graphene_vec4_t
  * @res: (out caller-allocates): return location for the result vector
  *
- * Negates the give #graphene_vec4_t.
+ * Negates the given #graphene_vec4_t.
  *
  * Since: 1.2
  */
